@@ -15,7 +15,7 @@
 """Tests for Chronicle rule functions."""
 
 import pytest
-from unittest.mock import Mock, patch, call
+from unittest.mock import Mock, patch
 from secops.chronicle.client import ChronicleClient
 from secops.chronicle.rule import (
     create_rule,
@@ -35,7 +35,13 @@ import json
 @pytest.fixture
 def chronicle_client():
     """Create a Chronicle client for testing."""
-    return ChronicleClient(customer_id="test-customer", project_id="test-project")
+    with patch("secops.auth.SecOpsAuth") as mock_auth:
+        mock_session = Mock()
+        mock_session.headers = {}
+        mock_auth.return_value.session = mock_session
+        return ChronicleClient(
+            customer_id="test-customer", project_id="test-project"
+        )
 
 
 @pytest.fixture
@@ -90,7 +96,7 @@ def test_create_rule(chronicle_client, mock_response):
 
         # Assert
         mock_post.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules",
             json={"text": "rule test {}"},
         )
         assert result == mock_response.json.return_value
@@ -121,7 +127,7 @@ def test_get_rule(chronicle_client, mock_response):
 
         # Assert
         mock_get.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules/{rule_id}"
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/{rule_id}"
         )
         assert result == mock_response.json.return_value
 
@@ -141,9 +147,11 @@ def test_get_rule_error(chronicle_client, mock_error_response):
 
 
 def test_list_rules(chronicle_client, mock_response):
-    """Test list_rules function."""
+    """Test list_rules function with single page."""
     # Arrange
-    mock_response.json.return_value = {"rules": [{"name": "rule1"}, {"name": "rule2"}]}
+    mock_response.json.return_value = {
+        "rules": [{"name": "rule1"}, {"name": "rule2"}]
+    }
 
     with patch.object(
         chronicle_client.session, "get", return_value=mock_response
@@ -153,13 +161,59 @@ def test_list_rules(chronicle_client, mock_response):
 
         # Assert
         mock_get.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules",
             params={"pageSize": 1000, "view": "FULL"},
         )
         assert result == mock_response.json.return_value
         assert len(result["rules"]) == 2
 
 
+def test_list_rules_pagination(chronicle_client):
+    """Test list_rules function with pagination."""
+
+    # Arrange
+    def get_mock_response(url, **kwargs):
+        params = kwargs.get("params", {})
+        if "pageToken" not in params:
+            # First page response
+            response = Mock()
+            response.status_code = 200
+            response.json.return_value = {
+                "rules": [{"name": "rule1"}, {"name": "rule2"}],
+                "nextPageToken": "page2token",
+            }
+            return response
+        elif params["pageToken"] == "page2token":
+            # Second page response
+            response = Mock()
+            response.status_code = 200
+            response.json.return_value = {
+                "rules": [{"name": "rule3"}, {"name": "rule4"}],
+            }
+            return response
+        else:
+            raise ValueError(f"Unexpected pageToken: {params.get('pageToken')}")
+
+    # Setup session.get to use our dynamic mock response function
+    with patch.object(chronicle_client.session, "get") as mock_get:
+        mock_get.side_effect = get_mock_response
+
+        # Act
+        result = list_rules(chronicle_client)
+
+        # Assert
+        assert mock_get.call_count == 2
+
+        # Verify the combined results
+        assert len(result["rules"]) == 4
+        assert [rule["name"] for rule in result["rules"]] == [
+            "rule1",
+            "rule2",
+            "rule3",
+            "rule4",
+        ]
+
+
 def test_list_rules_error(chronicle_client, mock_error_response):
     """Test list_rules function with error response."""
     # Arrange
@@ -187,7 +241,7 @@ def test_update_rule(chronicle_client, mock_response):
 
         # Assert
         mock_patch.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules/{rule_id}",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/{rule_id}",
             params={"update_mask": "text"},
             json={"text": rule_text},
         )
@@ -224,7 +278,7 @@ def test_delete_rule(chronicle_client, mock_response):
 
         # Assert
         mock_delete.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules/{rule_id}",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/{rule_id}",
             params={},
         )
         assert result == {}
@@ -259,7 +313,7 @@ def test_delete_rule_force(chronicle_client, mock_response):
 
         # Assert
         mock_delete.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules/{rule_id}",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/{rule_id}",
             params={"force": "true"},
         )
         assert result == {}
@@ -278,7 +332,7 @@ def test_enable_rule(chronicle_client, mock_response):
 
         # Assert
         mock_patch.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules/{rule_id}/deployment",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/{rule_id}/deployment",
             params={"update_mask": "enabled"},
             json={"enabled": True},
         )
@@ -298,7 +352,7 @@ def test_disable_rule(chronicle_client, mock_response):
 
         # Assert
         mock_patch.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules/{rule_id}/deployment",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules/{rule_id}/deployment",
             params={"update_mask": "enabled"},
             json={"enabled": False},
         )
@@ -323,7 +377,9 @@ def test_enable_rule_error(chronicle_client, mock_error_response):
 def test_search_rules(chronicle_client, mock_response):
     """Test search_rules function."""
     # Arrange
-    mock_response.json.return_value = {"rules": [{"name": "rule1"}, {"name": "rule2"}]}
+    mock_response.json.return_value = {
+        "rules": [{"name": "rule1"}, {"name": "rule2"}]
+    }
 
     with patch.object(
         chronicle_client.session, "get", return_value=mock_response
@@ -333,7 +389,7 @@ def test_search_rules(chronicle_client, mock_response):
 
         # Assert
         mock_get.assert_called_once_with(
-            f"{chronicle_client.base_url}/{chronicle_client.instance_id}/rules",
+            f"{chronicle_client.base_v1_url}/{chronicle_client.instance_id}/rules",
             params={"pageSize": 1000, "view": "FULL"},
         )
         assert result == mock_response.json.return_value
@@ -376,7 +432,9 @@ def test_run_rule_test(chronicle_client, mock_streaming_response):
         chronicle_client.session, "post", return_value=mock_response
     ) as mock_post:
         # Act
-        results = list(run_rule_test(chronicle_client, rule_text, start_time, end_time))
+        results = list(
+            run_rule_test(chronicle_client, rule_text, start_time, end_time)
+        )
 
         # Assert
         expected_url = f"{chronicle_client.base_url}/projects/{chronicle_client.project_id}/locations/{chronicle_client.region}/instances/{chronicle_client.customer_id}/legacy:legacyRunTestRule"
@@ -417,7 +475,9 @@ def test_run_rule_test_error(chronicle_client, mock_error_response):
     ):
         # Act & Assert
         with pytest.raises(APIError) as exc_info:
-            list(run_rule_test(chronicle_client, rule_text, start_time, end_time))
+            list(
+                run_rule_test(chronicle_client, rule_text, start_time, end_time)
+            )
 
         assert "Failed to test rule" in str(exc_info.value)
 
@@ -433,7 +493,11 @@ def test_run_rule_test_invalid_max_results(chronicle_client):
     with pytest.raises(ValueError) as exc_info:
         list(
             run_rule_test(
-                chronicle_client, rule_text, start_time, end_time, max_results=20000
+                chronicle_client,
+                rule_text,
+                start_time,
+                end_time,
+                max_results=20000,
             )
         )
 
@@ -443,7 +507,11 @@ def test_run_rule_test_invalid_max_results(chronicle_client):
     with pytest.raises(ValueError) as exc_info:
         list(
             run_rule_test(
-                chronicle_client, rule_text, start_time, end_time, max_results=-5
+                chronicle_client,
+                rule_text,
+                start_time,
+                end_time,
+                max_results=-5,
             )
         )
 
@@ -458,10 +526,14 @@ def test_run_rule_test_handles_exceptions(chronicle_client):
     rule_text = "rule test {}"
 
     with patch.object(
-        chronicle_client.session, "post", side_effect=Exception("Connection error")
+        chronicle_client.session,
+        "post",
+        side_effect=Exception("Connection error"),
     ):
         # Act & Assert
         with pytest.raises(APIError) as exc_info:
-            list(run_rule_test(chronicle_client, rule_text, start_time, end_time))
+            list(
+                run_rule_test(chronicle_client, rule_text, start_time, end_time)
+            )
 
         assert "Error testing rule" in str(exc_info.value)
